/* QCharSelect is a fork of KCharSelect from KDE 3.5.10, ported to pure Qt
 *
 * QCharSelect is Copyright (c) 2013 D. Michael McIntyre
 *
 * In KDE, KCharSelect the application was built around KCharSelect the library
 * widget.  QCharSelect extracts both components from KDE, moving the
 * application's core functionality into QCharSelectDialog, and moving the
 * library widget's core functionality into QCharSelectWidget, both of which
 * compile and link together to produce the qcharselect binary.
 *
 * The KCharSelect application was originally:
 *
 *     Copyright (C) 1999 Reginald Stadlbauer <reggie@kde.org>
 *
 * It was released under the GPL.
 *
 * The KCharSelect library widget was originally:
 *
 *     Copyright (C) 1999 Reginald Stadlbauer <reggie@kde.org>
 *
 * It was relesaed under the LGPL.  Under the terms of the LGPL, I am exercising
 * the option to release this derivative work under the GPL.  As such, the LGPL
 * notice has been removed from any source files where it was originally
 * present.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.  See the file COPYING included with this distribution for more
 * information.


    This file originally from:

    Rosegarden
    A MIDI and audio sequencer and musical notation editor.
    Copyright 2000-2013 the Rosegarden development team.

    Other copyrights also apply to some parts of this work.  Please
    see the AUTHORS file and individual file headers for details.

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of the
    License, or (at your option) any later version.  See the file
    COPYING included with this distribution for more information.
*/
#ifndef _RG_ACTIONFILECLIENT_H_
#define _RG_ACTIONFILECLIENT_H_

#include <QString>

class QAction;
class QMenu;
class QToolBar;
class QObject;

class ActionFileParser;

class ActionFileClient // base class for users of the ActionFileParser
{
public:
    /**
     * Find an action of the given name.  This function will always
     * return a valid Action pointer; if the action does not exist, a
     * DecoyAction will be returned.  Usage such as
     * findAction("action_name")->setChecked(true); is acceptable.
     */
    virtual QAction *findAction(QString actionName);
    virtual void enterActionState(QString stateName);
    virtual void leaveActionState(QString stateName);

    /**
     * Find a menu of the given object name.  If it does not exist,
     * this will currently return a null pointer -- beware the
     * inconsistency with the other methods here!
     */
    virtual QMenu *findMenu(QString menuName);

    /**
     * Find a toolbar of the given name, creating it if it does not
     * exist.
     */
    virtual QToolBar *findToolbar(QString toolbarName);

protected:
    ActionFileClient();
    virtual ~ActionFileClient();

    QAction *createAction(QString actionName, QString connection);
    QAction *createAction(QString actionName, QObject *target, QString connection);
    bool createGUI(QString rcname);
    friend class ActionCommandRegistry;

private:
    ActionFileParser *m_actionFileParser;
};

#endif
