/* QCharSelect is a fork of KCharSelect from KDE 3.5.10, ported to pure Qt
 *
 * QCharSelect is Copyright (c) 2013 D. Michael McIntyre
 *
 * In KDE, KCharSelect the application was built around KCharSelect the library
 * widget.  QCharSelect extracts both components from KDE, moving the
 * application's core functionality into QCharSelectDialoglog, and moving the
 * library widget's core functionality into QCharSelectWidget, both of which
 * compile and link together to produce the qcharselect binary.
 *
 * The KCharSelect application was originally:
 *
 *     Copyright (C) 1999 Reginald Stadlbauer <reggie@kde.org>
 *
 * It was released under the GPL.
 *
 * The KCharSelect library widget was originally:
 *
 *     Copyright (C) 1999 Reginald Stadlbauer <reggie@kde.org>
 *
 * It was relesaed under the LGPL.  Under the terms of the LGPL, I am exercising
 * the option to release this derivative work under the GPL.  As such, the LGPL
 * notice has been removed from any source files where it was originally
 * present.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.  See the file COPYING included with this distribution for more
 * information.
*/

#include "QCharSelectDialog.h"
#include "QCharSelectWidget.h"
#include "IconLoader.h"

#include <QDialog>
#include <QApplication>
#include <QSettings>
#include <QAction>

#include <stdlib.h>

/******************************************************************/
/* class QCharSelectDialog                                        */
/******************************************************************/

//==================================================================
QCharSelectDialog::QCharSelectDialog(const QChar &_chr,
                                     const QString &_font,
                                     int _tableNum,
                                     bool direction)
  : QMainWindow(),
      vChr(_chr),
      vFont(_font)
{
  setWindowTitle(tr("QCharSelect"));
  setIcon(IconLoader().loadPixmap("qcharselect32"));


  QWidget *mainWidget = new QWidget;
  setCentralWidget(mainWidget);
  layout = new QVBoxLayout;
  mainWidget->setLayout(layout);

  charSelect = new QCharSelectWidget(vFont, vChr, _tableNum);
  charSelect->resize(charSelect->sizeHint());
  connect(charSelect, SIGNAL(highlighted(const QChar &)),
          SLOT(charChanged(const QChar &)));
  connect(charSelect, SIGNAL(activated(const QChar &)),
          SLOT(add(const QChar &)));
  connect(charSelect, SIGNAL(fontChanged(const QString &)),
          SLOT(fontSelected(const QString &)));
  layout->addWidget(charSelect);

  QWidget *bbox = new QWidget;
  layout->addWidget(bbox);
  QHBoxLayout *bboxLayout = new QHBoxLayout;
  bbox->setLayout(bboxLayout);

  // Build line editor
  lined = new QLineEdit(mainWidget);
  lined->resize(lined->sizeHint());

  QFont font = lined->font();
  font.setFamily(vFont);
  lined->setFont(font);

  connect(lined,SIGNAL(textChanged(const QString &)),
      SLOT(lineEditChanged()));
  bboxLayout->addWidget(lined);

  // Build some buttons
  bHelp = new QPushButton(tr("Help"));
  connect(bHelp, SIGNAL(clicked()), this, SLOT(help()));
  bHelp->setFixedSize(bHelp->sizeHint());
  bboxLayout->addWidget(bHelp);

  QSpacerItem *space = new QSpacerItem(20, 20, QSizePolicy::Expanding);
  layout->addItem(space);

  bClear = new QPushButton(tr("Clear"));
//  bClear = new QPushButton(KStandardGuiItem::clear(), mainWidget);
  connect(bClear, SIGNAL(clicked()), this, SLOT(clear()));
  bClear->setFixedSize(bClear->sizeHint());
  bboxLayout->addWidget(bClear);

//  bClip = new QPushButton(KGuiItem(i18n("&To Clipboard"),
//            "editcopy"), mainWidget);
  bClip = new QPushButton(tr("&To Clipboard"));
  bClip->setFixedSize(bClip->sizeHint());
  connect(bClip, SIGNAL(clicked()), this, SLOT(toClip()));
  bboxLayout->addWidget(bClip);

/*  // Build menu
  KStandardAction::quit(this, SLOT(_exit()), actionCollection());
  
  new KAction(i18n("&To Clipboard"), "editcopy",
         KStandardShortcut::shortcut(KStandardShortcut::Copy), this, SLOT(toClip()), actionCollection(), "copy_clip");

  (void)new KAction(i18n("To Clipboard &UTF-8"), 0, this,
    SLOT(toClipUTF8()), actionCollection(), "copy_utf_8");
  (void)new KAction(i18n("To Clipboard &HTML"), 0, this,
      SLOT(toClipHTML()), actionCollection(), "copy_html");
 
  new KAction(i18n("&From Clipboard"), "editpaste",
         KStandardShortcut::shortcut(KStandardShortcut::Paste), this, SLOT(fromClip()), actionCollection(), "from_clip");
  (void)new KAction(i18n("From Clipboard UTF-8"), 0, this,
      SLOT(fromClipUTF8()), actionCollection(), "from_clip_utf8");
  
  i18n("From Clipboard HTML");      // Intended for future use
  
  KStandardAction::clear(this, SLOT(clear()), actionCollection(), "clear");
  (void)new KAction(i18n("&Flip"), 0, this,
      SLOT(flipText()), actionCollection(), "flip");
  (void)new KAction(i18n("&Alignment"), 0, this,
      SLOT(toggleEntryDirection()), actionCollection(), "alignment");*/
  
  charSelect->setFocus();

  entryDirection = direction;
  if(entryDirection)
    lined->setAlignment(Qt::AlignRight);
  else
    lined->setAlignment(Qt::AlignLeft);

//  setupGUI(Keys|StatusBar|Save|Create); // KDE idiom?
}

//==================================================================
void QCharSelectDialog::charChanged(const QChar &_chr)
{
  vChr = _chr;
}

//==================================================================
void QCharSelectDialog::fontSelected(const QString &_font)
{
  charSelect->setFont(_font);

  QFont font = lined->font();
  font.setFamily(_font);
  lined->setFont(font);

  vFont = _font;
}

//==================================================================
void QCharSelectDialog::add(const QChar &_chr)
{
  QString str;
  int cursorPos;

  charChanged(_chr);

  str       = lined->text();
  cursorPos = lined->cursorPosition();
  str.insert(cursorPos, vChr);
  lined->setText(str);
  cursorPos++;
  lined->setCursorPosition(cursorPos);
}

//==================================================================
void QCharSelectDialog::toClip()
{
  QClipboard *cb = QApplication::clipboard();
  // no setSelectionMode() or anything like it in Qt 4, nor any reference in porting guide
  //cb->setSelectionMode(true);
  cb->setText(lined->text());
  //cb->setSelectionMode(false);
  cb->setText(lined->text());
}

//==================================================================
// UTF-8 is rapidly becoming the favored 8-bit encoding for
// Unicode (iso10646-1).
//
void QCharSelectDialog::toClipUTF8()
{
  QClipboard *cb = QApplication::clipboard();
  QString str = lined->text();
  cb->setText(str.toUtf8());
}

//==================================================================
//  Put valid HTML 4.0 into the clipboard.  Valid ISO-8859-1 Latin 1
//  characters are left undisturbed.  Everything else, including the
//  "c0 control characters" often used by Windows, are clipped
//  as a HTML entity.
//
void QCharSelectDialog::toClipHTML()
{
    //!!!
/*  QClipboard *cb = QApplication::clipboard();
  QString input;
  QString html;
  QString tempstring;
  QChar   tempchar;
  uint i;

  input = lined->text();
  for (i = 0; i < input.length(); i++)
    {
      tempchar = input.at(i);
      if (tempchar.toLatin1().data() && ((tempchar.unicode() < 128) || (tempchar.unicode() >= 128+32)))
        {
          html.append(input.at(i));
        }
      else
        {
          html.append(tempstring.sprintf("&#x%x;", tempchar.unicode()));
        }
    }
  cb->setText(html); */
}

//==================================================================
//
void QCharSelectDialog::fromClip()
{
  QClipboard *cb = QApplication::clipboard();
  lined->setText(cb->text());
}

//==================================================================
// UTF-8 is rapidly becoming the favored 8-bit encoding for
// Unicode (iso10646-1).  This function is handy for decoding
// UTF-8 found in legacy applications, consoles, filenames, webpages,
// etc.
//
void QCharSelectDialog::fromClipUTF8()
{
  QClipboard *cb = QApplication::clipboard();
  QString str = cb->text();

  lined->setText(str.fromUtf8(str.toLatin1().data()));
}

//==================================================================
//  Reverse the text held in the line edit buffer.  This is crucial
//  for dealing with visual vs. logical representations of
//  right to left languages, and handy for working around all
//  manner of legacy character order issues.
//
void QCharSelectDialog::flipText()
{
  QString input;
  QString output;
  int i;

  input = lined->text();
  for (i = 0; i < input.length(); i++)
    {
      output.prepend(input.at(i));
    }
  lined->setText(output);
}

//==================================================================
void QCharSelectDialog::toggleEntryDirection()
{
    entryDirection ^= 1;
    if(entryDirection)
        lined->setAlignment(Qt::AlignRight);
    else
        lined->setAlignment(Qt::AlignLeft);
}

//==================================================================
void QCharSelectDialog::lineEditChanged()
{
    if(entryDirection)
      {
        if(lined->cursorPosition())
            lined->setCursorPosition(lined->cursorPosition() - 1);
      }
}

//==================================================================
void QCharSelectDialog::_exit()
{
    QSettings settings;
    settings.beginGroup("Options");
    settings.setValue("selected_font", vFont);
    settings.setValue("character", static_cast<int>(vChr.unicode()));
    settings.setValue("table", charSelect->tableNum());
    settings.setValue("text_direction", entryDirection);
    settings.endGroup();

    delete this;
    exit(0);
}

//==================================================================
void QCharSelectDialog::clear()
{
    lined->clear();
}

#include "QCharSelectDialog.moc"

