/* QCharSelect is a fork of KCharSelect from KDE 3.5.10, ported to pure Qt
 *
 * QCharSelect is Copyright (c) 2013 D. Michael McIntyre
 *
 * In KDE, KCharSelect the application was built around KCharSelect the library
 * widget.  QCharSelect extracts both components from KDE, moving the
 * application's core functionality into QCharSelectDialog, and moving the
 * library widget's core functionality into QCharSelectWidget, both of which
 * compile and link together to produce the qcharselect binary.
 *
 * The KCharSelect application was originally:
 *
 *     Copyright (C) 1999 Reginald Stadlbauer <reggie@kde.org>
 *
 * It was released under the GPL.
 *
 * The KCharSelect library widget was originally:
 *
 *     Copyright (C) 1999 Reginald Stadlbauer <reggie@kde.org>
 *
 * It was relesaed under the LGPL.  Under the terms of the LGPL, I am exercising
 * the option to release this derivative work under the GPL.  As such, the LGPL
 * notice has been removed from any source files where it was originally
 * present.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.  See the file COPYING included with this distribution for more
 * information.
*/

#ifndef QCharSelectDialog_h
#define QCharSelectDialog_h

#include "QCharSelectWidget.h"

#include <QLayout>
#include <QLineEdit>
#include <QClipboard>
#include <QPushButton>
#include <QApplication>
#include <QMainWindow>

/******************************************************************/
/* class QCharSelectDialog                                        */
/******************************************************************/

class QCharSelectDialog : public QMainWindow
{
  Q_OBJECT

public:
  QCharSelectDialog(const QChar &_chr,
                    const QString &_font,
                    int _tableNum,
                    bool direction);

  static bool selectChar(QString &_font,
                         QChar &_chr,
                         int _tableNum);

  int chr() { return static_cast<unsigned short>(vChr.unicode()); } //!!!
  QString font() { return vFont; }

protected:
  void closeEvent(QCloseEvent *) { _exit(); }

  QVBoxLayout       *layout;
  QCharSelectWidget *charSelect;
  QLineEdit         *lined;
  QPushButton       *bClip,*bClear;
  QPushButton       *bHelp;

  QChar vChr;
  QString vFont;
  int pointSize;
  bool entryDirection;

protected slots:
  void charChanged(const QChar &_chr);
  void fontSelected(const QString &_font);
  void add() { add(vChr); }
  void add(const QChar &_chr);
  void toClip();
  void toClipUTF8();
  void toClipHTML();
  void fromClip();
  void fromClipUTF8();
  void flipText();
  void toggleEntryDirection();
  void lineEditChanged();
  void _exit();
  void clear();
//  void help();

};

#endif
